#ifndef __include_appconf_h__
#define __include_appconf_h__

#include <stdarg.h>

enum conf_type {
	conf_type_any,

	conf_type_bool,
	conf_type_int,
	conf_type_float,
	conf_type_str,
	conf_type_range,
	conf_type_enum,
	conf_type_strarray,

	conf_type_group,
};
struct conf_valid_range_data {
	int lower_limit, upper_limit;
};
struct conf_value;
struct conf_enum_list;
struct conf_group;

typedef int (*conf_foreach_func)(struct conf_group *g, const char *name,
		struct conf_value *v, void *d);

struct conf_enum_list {
	int num;
	const char *str;
};
#define conf_free_val(cv)	do { } while(0)	

/* conf_value_to_*
 * These functions all convert a conf_value structure into a particular
 * type, doing there best to do the conversion as specified. 
 *
 * Eg if the user specified "true" as a setting, the following are 
 * the answers each function would return:
 * conf_value_to_str: sucess an buffer contains "true"
 * conf_value_to_bool: success and b=1
 * conf_value_to_int: failure 
 * conf_value_to_float: failure
 * conf_value_to_enum: If the enum list has an entry for "true" then 
 *  			return that, otherwise returns failure
 * conf_value_to_strarrary: A single arrary with "true" in it.
 * conf_value_to_group: failure. 
 *
 * If NULL is passed as the conf_value, then failure is returned.
 * This makes it possible to do:
 * cp = conf_value_to_str_const(conf_get_value(NULL,"/test/"));
 *
 * conv_value_to_strarrary is the only function that takes an 
 * extra parameter being pool. It uses this parameter to allocate
 * enough space to do the string arrary.
 *
 * You can access a group by doing:
 *  struct conf_value *cv;
 *  struct conf_group *cg;
 *  cv = conf_get_value(NULL,"/name/of/group");
 *  if (cv == NULL) 
 *  	return -1; // No such group
 *  cg = conf_value_to_group(cv);
 *  if (cg == NULL) 
 *  	return -1; // Entry wasn't a group.
 * 
 * The conf_value_to_* functions all return 0 on success or -1
 * on failure.
 */

const char *conf_value_to_str_const(struct conf_value *val);
int conf_value_to_bool(struct conf_value *val, int *b);
int conf_value_to_str(struct conf_value *val, char *buffer, size_t size);
int conf_value_to_int(struct conf_value *val, int *i);
int conf_value_to_float(struct conf_value *val, double *f);
int conf_value_to_enum(struct conf_value *val, int *v, 
		struct conf_enum_list *el);
int conf_value_to_strarray(POOL p,struct conf_value *val, char ***a);

struct conf_group *conf_value_to_group(struct conf_value *val);


/* This function is similar to the conf_value_to_* functions above except
 * it converts one conf_type to another. This function isn't of much
 * use, since the other conf_value_to_* functions return the conf_value
 * as a usable type, and call this function internaly
 */
struct conf_value *conf_value_to_type(POOL p,struct conf_value *cv, 
		enum conf_type type);

/* This function returns the name of the conf_group */
const char *conf_group_name(struct conf_group *cg);

/* This function converts a conf_type to a string representing the
 * name of the type
 */
const char *conf_type_to_str(enum conf_type type);

int conf_value_foreach(struct conf_group *group, conf_foreach_func func);
int conf_foreach(struct conf_group *cg, conf_foreach_func func, void *appdata);

/* conf_get_value(pool, valuepath, ...)
 * conf_get_value(pool, valuepath, va_list)
 * conf_get_value_str(pool, valuepath)
 *
 * These functions all return a conf_value structure representing the
 * value of the configuration entry in ircfs.conf specified by valuepath.
 * If no such entry exists, NULL is returned (You can pass NULL to
 * the conf_value_to_* functions and they will simply return -1).
 *
 * If the pool parameter is NULL, then the temporary pool is used for
 * allocations.
 *
 * Returns NULL on failure (the entry doesn't exist, or internal failure)
 * and a conf_value structure on success.
 *
 * See also: conf_value_to_* functions
 */
struct conf_value *conf_get_value(POOL p, const char *valuepath, ...);	
struct conf_value *conf_get_value_v(POOL p, const char *valuepath, va_list vl);	
struct conf_value *conf_get_value_str(POOL p, const char *valuepath);	


/* Returns the conf_value subsetting in group cg named name */
struct conf_value *conf_group_value(POOL p, struct conf_group *cg, 
		const char *name);
/* Returns the nth conf_value subsetting in group cg */
struct conf_value *conf_group_value_n(POOL p, struct conf_group *cg, 
		int num);





struct conf_value *conf_new_enum(POOL p, int i);
struct conf_value *conf_new_int(POOL p, int i);
struct conf_value *conf_new_str(POOL p, const char *str);
struct conf_value *conf_new_bool(POOL p, int i);
struct conf_value *conf_new_float(POOL p, double f);
struct conf_value *conf_new_group_val(POOL p, struct conf_group *cg);
struct conf_value *conf_new_strarray(POOL p, char **a);

/* appconf_load_config(filename)
 * appconf_loadfile(filename)
 *
 * These two functions both load a configuration file into memory,
 * however appconf_load_config searches for the filename in the
 * usual places (~/.ircfs, /etc/ircfs/, etc)
 * Where as loadfile simply loads the specified file, without searching
 * for it in other places
 */
int appconf_load_config(const char *filename);
int appconf_loadfile(const char *filename);

/* Produce a standard debug message stating a required value (path)
 * is not found.
 */
void conf_missing(DEBUG h,const char *path);
/* Produce a stardard debug message stating a configuration value (path)
 * is invalid, and should be of the type type.
 */
void conf_invalid(DEBUG h,const char *path, enum conf_type type);
/* Dumps a copy of a string array to the debug handler
 */
void conf_debug_strarray(DEBUG h, char **strarray);
/* Parses an enum list, and sets result to contain the number corisponding
 * to the enum entry in the list for the value str. Returns 0 if sucessful
 * and -1 if not found.
 */
int parse_enum_list(const char *str,struct conf_enum_list *el, int *result);
#endif
